@@ -75,7 +75,8 @@ typedef struct data
      unsigned int mode_flags;    /**< flags associated with browser calls */
      char *mimetype;
      char *href;                 /**< If QT this is set to handle special case */
-     char *mms;                  /**< Is set if Browser cant handle protocol */ 
+     char *url;                  /**< The URL */ 
+     char browserCantHandleIt;   /**< Is set if browser cant handle protocol */
      char *urlFragment;
 
      int          tmpFileFd;     /**< File descriptor of temp file */
@@ -1543,14 +1544,12 @@ static int match_url(const char * matchStr, const char * url)
  *                       plugin
  * @param[in] streamOnly If true select entry sith stream set only
  * @param[in] c          Pointer to command structure to match against
- * @param[in] url        The url
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
 __inline
-static int match_command(data_t * const THIS, int streamOnly, 
-                         command_t *c, const char * url)
+static int match_command(data_t * const THIS, int streamOnly, command_t *c)
 {
 #define MODE_MASK (H_NOEMBED | H_EMBED | H_LINKS)
 
@@ -1581,10 +1580,10 @@ static int match_command(data_t * const THIS, int streamOnly,
 
      if(c->fmatchStr)
      {
-          if(!match_url(c->fmatchStr, url))
+          if(!match_url(c->fmatchStr, THIS->url))
           {
                D("fmatch mismatch: url '%s' doesnt have '%s'\n", 
-                                              url, c->fmatchStr);
+                                              THIS->url, c->fmatchStr);
                return 0;
           }
      }
@@ -1632,14 +1631,12 @@ static int match_mime_type(data_t * const THIS, mimetype_t *m)
  * @param[in] h          Pointer to handler to match against
  * @param[in] THIS       Pointer to data associated with this instance of plugin
  * @param[in] streamOnly If True select only entry with stream flag
- * @param[in] url        The URL 
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
 __inline
-static int match_handler(handler_t *h, data_t * const THIS, 
-                         int streamOnly, const char *url)
+static int match_handler(handler_t *h, data_t * const THIS, int streamOnly)
 {
      mimetype_t *m;
      command_t *c;
@@ -1655,7 +1652,7 @@ static int match_handler(handler_t *h, data_t * const THIS,
                c = h->cmds;
                while(c)
 	       {
-		    if (match_command(THIS, streamOnly, c, url))
+		    if (match_command(THIS, streamOnly, c))
 		    {
 			 THIS->cmd_flags = c->flags;
 			 THIS->command = c->cmd;
@@ -1676,12 +1673,11 @@ static int match_handler(handler_t *h, data_t * const THIS,
  *
  * @param[in] THIS        Pointer to plugin instance data
  * @param[in] streamOnly  If true select only the command with stream flag
- * @param[in] url         The URL  
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
-static int find_command(data_t * const THIS, int streamOnly, const char * url)
+static int find_command(data_t * const THIS, int streamOnly)
 {
      handler_t * h;
 
@@ -1697,7 +1693,7 @@ static int find_command(data_t * const THIS, int streamOnly, const char * url)
      h = handlers;
      while(h)
      {
-	  if (match_handler(h, THIS, streamOnly, url))
+	  if (match_handler(h, THIS, streamOnly))
 	  {
 	       D("Command found.\n");
 	       return 1;
@@ -2256,14 +2252,14 @@ NPError NPP_New(NPMIMEType pluginType,
 
      memset((void *)THIS, 0, sizeof(data_t));
 
-     THIS->windata.window = 0;
-     THIS->display = NULL;
+     /* Only initialise the non-zero fields */
      THIS->pid = -1;
      THIS->commsPipeFd = -1;
      THIS->repeats = 1;
      THIS->autostart = 1;
      THIS->autostartNotSeen = 1;
      THIS->tmpFileFd = -1;
+
      if(mode == NP_EMBED)
      {
          THIS->mode_flags = H_EMBED;
@@ -2400,6 +2396,8 @@ NPError NPP_New(NPMIMEType pluginType,
 	       return NPERR_GENERIC_ERROR;
           }
 
+          THIS->url = url;
+
           /* Mozilla does not support the following protocols directly and
            * so it never calls NPP_NewStream for these protocols */
 	  if(   (strncmp(url, "mms://", 6) == 0)        
@@ -2411,12 +2409,19 @@ NPError NPP_New(NPMIMEType pluginType,
 	  {
 	       D("Detected MMS -> url=%s\n", url);
 
-               THIS->mms = url;
+               THIS->browserCantHandleIt = true;
+               find_command(THIS,1);            /* Needs to be done early! so xembed 
+                                                       flag is correctly set*/
+
+
                /* The next call from browser will be NPP_SetWindow() & 
                 * NPP_NewStream will never be called */
 	  }
           else 
           {
+               find_command(THIS,0);           /* Needs to be done early so xembed
+                                                      flag is correctly set*/
+
                /* For protocols that Mozilla does support, sometimes
                 * the browser will call NPP_NewStream straight away, some
                 * times it wont (depends on the nature of the tag). So that
@@ -2511,7 +2516,7 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 
 	  NPN_MemFree(THIS->mimetype);
 	  THIS->href = NULL;
-	  THIS->mms = NULL;
+	  THIS->url = NULL;
 
           NPN_MemFree(THIS->urlFragment);
           THIS->urlFragment = NULL;
@@ -2530,8 +2535,7 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
  * Check that no child is already running before forking one.
  *
  * @param[in] THIS     Pointer to the plugin instance data
- * @param[in] fname    The filename of the embedded object or NULL if using a
- *                     pipe
+ * @param[in] fname    The filename of the embedded object
  *
  * @return Nothing
  *
@@ -2777,6 +2781,7 @@ NPError NPP_NewStream(NPP instance,
      char fileName[SMALL_BUFFER_SIZE];
      char * savedMimetype = NULL;
      data_t * THIS;
+     char refind_command = 0;
 
      D("NPP_NewStream() - instance=%p\n", instance);
 
@@ -2787,8 +2792,8 @@ NPError NPP_NewStream(NPP instance,
 
      THIS = instance->pdata;
 
-     /* Looks like browser can handle this stream so we can clear the mms flag */
-     THIS->mms = NULL;
+     /* Looks like browser can handle this stream so we can clear the flag */
+     THIS->browserCantHandleIt = 0;
 
      if((THIS->pid != -1) || (THIS->tmpFileFd >= 0))
      {
@@ -2803,10 +2808,24 @@ NPError NPP_NewStream(NPP instance,
      if((THIS->href != NULL) && (safeURL(THIS->href)))
      {
 	  D("Replacing SRC with HREF... \n");
-	  stream->url = THIS->href;
-     }
 
-     D("Url is %s (seekable=%d)\n", stream->url, seekable);
+          if((THIS->url == 0) || (strcmp(THIS->href, THIS->url) != 0))
+          {
+              /* URL has changed */
+              D("URL has changed to %s\n", THIS->href);
+              THIS->url = THIS->href;
+              refind_command = 1;
+          }
+     }
+     else if((THIS->url == 0) || (strcmp(stream->url, THIS->url) != 0))
+     {
+          /* URL has changed */
+          D("URL has changed to %s\n", stream->url);
+          THIS->url = stream->url;
+          refind_command = 1;
+     }
+    
+     D("Url is %s (seekable=%d)\n", THIS->url, seekable);
 
      /* Ocassionally the MIME type here is different to that passed to the
       * NEW function - this is because of either badly configure web server
@@ -2818,29 +2837,29 @@ NPError NPP_NewStream(NPP instance,
      if(strcmp(type, THIS->mimetype) != 0)
      {
           D("Mismatching mimetype reported, originally was \'%s\' now '\%s' "
-            "for url %s\n", THIS->mimetype, type, stream->url);
+                          "for url %s\n", THIS->mimetype, type, THIS->url);
           savedMimetype = THIS->mimetype;
           THIS->mimetype = NP_strdup(type);
 
-          if(!find_command(THIS, 0, stream->url))
+          if(!find_command(THIS, 0))
           {
                 NPN_MemFree(THIS->mimetype);
                 THIS->mimetype = savedMimetype;
-                find_command(THIS, 0, stream->url);
+                find_command(THIS, 0);
           }
           else
           {
               NPN_MemFree(savedMimetype);
           }
      }
-     else
+     else if(refind_command)
      {
+          find_command(THIS, 0);
           D("Mime type %s\n", type);
-          find_command(THIS, 0, stream->url);
      }
 
      /* Extract from the URL the various additional information */
-     parseURL(THIS, stream->url, fileName, sizeof(fileName)/sizeof(char)-1);
+     parseURL(THIS, THIS->url, fileName, sizeof(fileName)/sizeof(char)-1);
 
      /* Extract the fileName from HTTP headers, overide URL fileName */
      parseHeaders(THIS, stream->headers, fileName, sizeof(fileName)/sizeof(char)-1);
@@ -2871,7 +2890,7 @@ NPError NPP_NewStream(NPP instance,
      }
      else
      {
-         new_child(instance, stream->url);
+         new_child(instance, THIS->url);
      }
 
      *stype = NP_NORMAL;
@@ -2988,31 +3007,33 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
      THIS->displayname = XDisplayName(DisplayString(THIS->display));
      THIS->windata = *window;
 
-     if (THIS->mms)
+     if ((THIS->url) && (THIS->browserCantHandleIt))
      {
-          /* Can only use streaming commands, as Mozilla cannot handle
-           * these types (mms) of urls */
-          if (!find_command(THIS, 1, THIS->mms))
+          if(THIS->command == 0)
           {
-	       THIS->mms = NULL;
-               if(errMsg)
+               /* Can only use streaming commands, as Mozilla cannot handle
+                * these types (mms) of urls */
+               if (!find_command(THIS, 1))
                {
-                   NPN_Status(instance, errMsg);
-                   errMsg = NULL;
-               }
-               else
-               {
-	           NPN_Status(instance, 
+                    if(errMsg)
+                    {
+                         NPN_Status(instance, errMsg);
+                         errMsg = NULL;
+                    }
+                    else
+                    {
+	                 NPN_Status(instance, 
                           "MozPlugger: No appropriate application found.");
+                    }
+ 	            return NPERR_GENERIC_ERROR;
                }
- 	       return NPERR_GENERIC_ERROR;
           }
 
           /* Extract from the URL the various additional information */
-          parseURL(THIS, THIS->mms, 0, 0);
+          parseURL(THIS, THIS->url, 0, 0);
 
-	  new_child(instance, THIS->mms);
-	  THIS->mms = NULL;
+	  new_child(instance, THIS->url);
+          THIS->url = NULL;                 /* Stops new_child from being called again */
 	  return NPERR_NO_ERROR;
      }
 
